"""Python library to connect deCONZ and Home Assistant to work together."""

import logging

from .deconzdevice import DeconzDevice

_LOGGER = logging.getLogger(__name__)


class DeconzSensor(DeconzDevice):
    """deCONZ sensor representation.

    Dresden Elektroniks documentation of sensors in deCONZ
    http://dresden-elektronik.github.io/deconz-rest-doc/sensors/
    """

    DECONZ_TYPE = '/sensors/'

    BINARY = None
    ZHATYPE = set()

    SENSOR_CLASS = None
    SENSOR_ICON = None
    SENSOR_UNIT = None

    @property
    def battery(self):
        """The battery status of the sensor."""
        return self.raw['config'].get('battery')

    @property
    def ep(self):
        """The Endpoint of the sensor."""
        return self.raw.get('ep')

    @property
    def lowbattery(self):
        """Low battery."""
        return self.raw['state'].get('lowbattery')

    @property
    def on(self):
        """Declare if the sensor is on or off."""
        return self.raw['config'].get('on')

    @property
    def reachable(self):
        """Declare if the sensor is reachable."""
        return self.raw['config'].get('reachable', True)

    @property
    def tampered(self):
        """Tampered."""
        return self.raw['state'].get('tampered')

    @property
    def secondary_temperature(self):
        """Extra temperature available on some Xiaomi devices."""
        if 'temperature' not in self.raw['config']:
            return None

        return Temperature.convert_temperature(
            self.raw['config'].get('temperature'))


class Alarm(DeconzSensor):
    """Alarm sensor."""

    BINARY = False
    ZHATYPE = ('ZHAAlarm',)

    SENSOR_CLASS = 'motion'

    @property
    def state(self):
        """Main state of sensor."""
        return self.alarm

    @property
    def alarm(self):
        """Alarm."""
        return self.raw['state'].get('alarm')


class CarbonMonoxide(DeconzSensor):
    """Carbon monoxide sensor."""

    BINARY = True
    ZHATYPE = ('ZHACarbonMonoxide',)

    SENSOR_CLASS = 'carbon_monoxide'

    @property
    def state(self):
        """Main state of sensor."""
        return self.is_tripped

    @property
    def carbonmonoxide(self):
        """Carbon monoxide detected."""
        return self.raw['state'].get('carbonmonoxide')

    @property
    def is_tripped(self):
        """Sensor is tripped."""
        return self.carbonmonoxide


class Consumption(DeconzSensor):
    """Power consumption sensor."""

    BINARY = False
    ZHATYPE = ('ZHAConsumption',)

    SENSOR_UNIT = 'kWh'

    @property
    def state(self):
        """Main state of sensor."""
        if self.consumption is None:
            return None

        return float(self.consumption/1000)

    @property
    def consumption(self):
        """Consumption."""
        return self.raw['state'].get('consumption')

    @property
    def power(self):
        """Power."""
        return self.raw['state'].get('power')


class Daylight(DeconzSensor):
    """Daylight sensor built into deCONZ software.

    Strings from daylight.h at
    https://github.com/dresden-elektronik/deconz-rest-plugin.
    Also has a 'daylight' boolean.
    Has no 'reachable' config parameter, so set sensor reachable True here.
    """

    BINARY = False
    ZHATYPE = ('Daylight',)

    SENSOR_CLASS = 'daylight'
    SENSOR_ICON = 'mdi:white-balance-sunny'

    @property
    def state(self):
        """Main state of sensor."""
        return self.status

    @property
    def configured(self):
        """Is daylight sensor configured."""
        return self.raw['config'].get('configured')

    @property
    def daylight(self):
        """True if daylight, false if not."""
        return self.raw['state'].get('daylight')

    @property
    def status(self):
        """Return the daylight status string."""
        status = self.raw['state'].get('status')
        if status == 100:
            return 'nadir'
        if status == 110:
            return 'night_end'
        if status == 120:
            return 'nautical_dawn'
        if status == 130:
            return 'dawn'
        if status == 140:
            return 'sunrise_start'
        if status == 150:
            return 'sunrise_end'
        if status == 160:
            return 'golden_hour_1'
        if status == 170:
            return 'solar_noon'
        if status == 180:
            return 'golden_hour_2'
        if status == 190:
            return 'sunset_start'
        if status == 200:
            return 'sunset_end'
        if status == 210:
            return 'dusk'
        if status == 220:
            return 'nautical_dusk'
        if status == 230:
            return 'night_start'
        return 'unknown'

    @property
    def sunriseoffset(self):
        """Sunrise offset."""
        return self.raw['config'].get('sunriseoffset')

    @property
    def sunsetoffset(self):
        """Sunset offset."""
        return self.raw['config'].get('sunsetoffset')


class Fire(DeconzSensor):
    """Fire sensor."""

    BINARY = True
    ZHATYPE = ('ZHAFire',)

    SENSOR_CLASS = 'smoke'

    @property
    def state(self):
        """Main state of sensor."""
        return self.is_tripped

    @property
    def fire(self):
        """Fire detected."""
        return self.raw['state'].get('fire')

    @property
    def is_tripped(self):
        """Sensor is tripped."""
        return self.fire


class GenericFlag(DeconzSensor):
    """Generic flag sensor."""

    BINARY = True
    ZHATYPE = ('CLIPGenericFlag',)

    @property
    def state(self):
        """Main state of sensor."""
        return self.flag

    @property
    def flag(self):
        """Flag status."""
        return self.raw['state'].get('flag')

    @property
    def is_tripped(self):
        """Sensor is tripped."""
        return self.flag


class GenericStatus(DeconzSensor):
    """Generic status sensor."""

    BINARY = False
    ZHATYPE = ('CLIPGenericStatus',)

    @property
    def state(self):
        """Main state of sensor."""
        return self.status

    @property
    def status(self):
        """Status."""
        return self.raw['state'].get('status')


class Humidity(DeconzSensor):
    """Humidity sensor."""

    BINARY = False
    ZHATYPE = ('ZHAHumidity', 'CLIPHumidity')

    SENSOR_CLASS = 'humidity'
    SENSOR_UNIT = '%'

    @property
    def state(self):
        """Main state of sensor."""
        if self.humidity is None:
            return None

        return round(float(self.humidity) / 100, 1)

    @property
    def humidity(self):
        """Humidity level."""
        return self.raw['state'].get('humidity')


class LightLevel(DeconzSensor):
    """Light level sensor."""

    BINARY = False
    ZHATYPE = ('ZHALightLevel', 'CLIPLightLevel')

    SENSOR_CLASS = 'illuminance'
    SENSOR_UNIT = 'lux'

    @property
    def state(self):
        """Main state of sensor."""
        if self.lightlevel is None:
            return None

        return round(10 ** (float(self.lightlevel - 1) / 10000), 1)

    @property
    def dark(self):
        """If the area near the sensor is light or not."""
        return self.raw['state'].get('dark')

    @property
    def daylight(self):
        """Daylight."""
        return self.raw['state'].get('daylight')

    @property
    def lightlevel(self):
        """Light level."""
        return self.raw['state'].get('lightlevel')

    @property
    def lux(self):
        """Lux."""
        return self.raw['state'].get('lux')

    @property
    def tholddark(self):
        """Threshold to hold dark."""
        return self.raw['config'].get('tholddark')

    @property
    def tholdoffset(self):
        """Offset for threshold to hold dark."""
        return self.raw['config'].get('tholdoffset')


class OpenClose(DeconzSensor):
    """Door/Window sensor."""

    BINARY = True
    ZHATYPE = ('ZHAOpenClose', 'CLIPOpenClose')

    SENSOR_CLASS = 'opening'

    @property
    def state(self):
        """Main state of sensor."""
        return self.is_tripped

    @property
    def is_tripped(self):
        """Sensor is tripped."""
        return self.open

    @property
    def open(self):
        """Door open."""
        return self.raw['state'].get('open')


class Power(DeconzSensor):
    """Power sensor."""

    BINARY = False
    ZHATYPE = ('ZHAPower',)

    SENSOR_UNIT = 'W'

    @property
    def state(self):
        """Main state of sensor."""
        return self.power

    @property
    def current(self):
        """Current."""
        return self.raw['state'].get('current')

    @property
    def power(self):
        """Power."""
        return self.raw['state'].get('power')

    @property
    def voltage(self):
        """Voltage."""
        return self.raw['state'].get('voltage')


class Presence(DeconzSensor):
    """Presence detector."""

    BINARY = True
    ZHATYPE = ('ZHAPresence', 'CLIPPresence')

    SENSOR_CLASS = 'motion'

    @property
    def state(self):
        """Main state of sensor."""
        return self.is_tripped

    @property
    def dark(self):
        """If the area near the sensor is light or not."""
        return self.raw['state'].get('dark')

    @property
    def duration(self):
        """Minimum duration which presence will be true."""
        return self.raw['config'].get('duration')

    @property
    def is_tripped(self):
        """Sensor is tripped."""
        return self.presence

    @property
    def presence(self):
        """Motion detected."""
        return self.raw['state'].get('presence')


class Pressure(DeconzSensor):
    """Pressure sensor."""

    BINARY = False
    ZHATYPE = ('ZHAPressure', 'CLIPPressure')

    SENSOR_CLASS = 'pressure'
    SENSOR_ICON = 'mdi:gauge'
    SENSOR_UNIT = 'hPa'

    @property
    def state(self):
        """Main state of sensor."""
        return self.pressure

    @property
    def pressure(self):
        """Pressure."""
        return self.raw['state'].get('pressure')


class Switch(DeconzSensor):
    """Switch sensor."""

    BINARY = False
    ZHATYPE = ('ZHASwitch', 'ZGPSwitch', 'CLIPSwitch')

    @property
    def state(self):
        """Main state of switch."""
        return self.buttonevent

    @property
    def buttonevent(self):
        """Button press."""
        return self.raw['state'].get('buttonevent')


class Temperature(DeconzSensor):
    """Temperature sensor."""

    BINARY = False
    ZHATYPE = ('ZHATemperature', 'CLIPTemperature')

    SENSOR_CLASS = 'temperature'
    SENSOR_ICON = 'mdi:thermometer'
    SENSOR_UNIT = '°C'

    @property
    def state(self):
        """Main state of sensor."""
        return self.temperature

    @property
    def temperature(self):
        """Temperature."""
        return self.convert_temperature(self.raw['state'].get('temperature'))

    @staticmethod
    def convert_temperature(temperature):
        """Convert temperature to celsius"""
        if temperature is None:
            return None

        return round(float(temperature) / 100, 1)


class Thermostat(Temperature):
    """Thermostat "sensor"."""

    BINARY = False
    ZHATYPE = ('ZHAThermostat', 'CLIPThermostat')

    @property
    def heatsetpoint(self):
        """Heating setpoint"""
        return self.convert_temperature(self.raw['config'].get('heatsetpoint'))

    @property
    def locked(self):
        """"""
        return self.raw['config'].get('locked')

    @property
    def mode(self):
        """Thermostat mode; "off", "heat", and "auto"."""
        return self.raw['config'].get('mode')

    @property
    def offset(self):
        """Temperature offset."""
        return self.raw['config'].get('offset')

    @property
    def state_on(self):
        """Declare if the sensor is on or off."""
        return self.raw['state'].get('on')

    @property
    def valve(self):
        """How open the valve is."""
        return self.raw['state'].get('valve')


class Vibration(DeconzSensor):
    """Vibration sensor."""

    BINARY = True
    ZHATYPE = ('ZHAVibration',)

    SENSOR_CLASS = 'motion'

    @property
    def state(self):
        """Main state of sensor."""
        return self.is_tripped

    @property
    def is_tripped(self) -> bool:
        """Sensor is tripped."""
        return self.vibration

    @property
    def orientation(self) -> list:
        """Orientation."""
        return self.raw['state'].get('orientation')

    @property
    def sensitivity(self) -> int:
        """Configured sensitivity"""
        return self.raw['config'].get('sensitivity')

    @property
    def sensitivitymax(self) -> int:
        """Configured max sensitivity."""
        return self.raw['config'].get('sensitivitymax')

    @property
    def tiltangle(self) -> int:
        """Tilt angle."""
        return self.raw['state'].get('tiltangle')

    @property
    def vibration(self) -> bool:
        """Vibration."""
        return self.raw['state'].get('vibration')

    @property
    def vibrationstrength(self) -> int:
        """Strength of vibration."""
        return self.raw['state'].get('vibrationstrength')


class Water(DeconzSensor):
    """Water sensor."""

    BINARY = True
    ZHATYPE = ('ZHAWater',)

    SENSOR_CLASS = 'moisture'

    @property
    def state(self):
        """Main state of sensor."""
        return self.is_tripped

    @property
    def is_tripped(self):
        """Sensor is tripped."""
        return self.water

    @property
    def water(self):
        """Water detected."""
        return self.raw['state'].get('water')


SENSOR_CLASSES = (
    Alarm, CarbonMonoxide, Consumption, Daylight, Fire, GenericFlag,
    GenericStatus, Humidity, LightLevel, OpenClose, Power, Presence, Pressure,
    Switch, Temperature, Thermostat, Vibration, Water)


def create_sensor(sensor_id, raw, loop, async_set_state_callback):
    """Simplify creating sensor by not needing to know type."""
    for sensor_class in SENSOR_CLASSES:
        if raw['type'] in sensor_class.ZHATYPE:
            return sensor_class(
                sensor_id, raw, loop, async_set_state_callback)


def supported_sensor(raw):
    """Check if sensor is supported by pydeconz."""
    for sensor_class in SENSOR_CLASSES:
        if raw['type'] in sensor_class.ZHATYPE:
            return True

    _LOGGER.info('Unsupported sensor type %s (%s)',
                 raw['type'], raw['name'])
    return False



"""Python library to connect deCONZ and Home Assistant to work together."""

import logging

from .deconzdevice import DeconzDevice

_LOGGER = logging.getLogger(__name__)


class DeconzLight(DeconzDevice):
    """deCONZ light representation.

    Dresden Elektroniks documentation of lights in deCONZ
    http://dresden-elektronik.github.io/deconz-rest-doc/lights/
    """

    DECONZ_TYPE = '/lights/'

    @property
    def state(self):
        """True if the light is on."""
        return self.raw['state'].get('on')

    @property
    def alert(self):
        """Temporary alert effect.

        Following values are possible:
        none - light is not performing an alert
        select - light is blinking a short time
        lselect - light is blinking a longer time
        """
        return self.raw['state'].get('alert')

    @property
    def brightness(self):
        """Brightness of the light.

        Depending on the light type 0 might not mean visible "off"
        but minimum brightness.
        """
        return self.raw['state'].get('bri')

    @property
    def ct(self):
        """Mired color temperature of the light. (2000K - 6500K)."""
        return self.raw['state'].get('ct')

    @property
    def hue(self):
        """Color hue of the light.

        The hue parameter in the HSV color model is between 0°-360°
        and is mapped to 0..65535 to get 16-bit resolution.
        """
        return self.raw['state'].get('hue')

    @property
    def sat(self):
        """Color saturation of the light.

        There 0 means no color at all and 255 is the greatest saturation
        of the color.
        """
        return self.raw['state'].get('sat')

    @property
    def xy(self):
        """CIE xy color space coordinates as array [x, y] of real values (0..1)."""
        if 'xy' not in self.raw['state'] or \
                self.raw['state']['xy'] == (None, None):
            return None

        x, y = self.raw['state']['xy']

        if x > 1:
            x = x / 65555

        if y > 1:
            y = y / 65555

        return (x, y)

    @property
    def colormode(self):
        """The current color mode of the light.

        hs - hue and saturation
        xy - CIE xy values
        ct - color temperature
        """
        return self.raw['state'].get('colormode')

    @property
    def hascolor(self) -> bool:
        """Tells if light has color support."""
        return self.raw['state'].get('hascolor')

    @property
    def effect(self):
        """Effect of the light.

        none - no effect
        colorloop
        """
        return self.raw['state'].get('effect')

    @property
    def reachable(self):
        """True if the light is reachable and accepts commands."""
        return self.raw['state']['reachable']



"""Python library to connect deCONZ and Home Assistant to work together."""

import logging
from pprint import pformat

from .errors import BridgeBusy

_LOGGER = logging.getLogger(__name__)


class DeconzDevice:
    """deCONZ resource base representation.

    Dresden Elektroniks REST API documentation
    http://dresden-elektronik.github.io/deconz-rest-doc/
    """

    DECONZ_TYPE = ''

    def __init__(self, device_id, raw, loop, async_set_callback):
        """Set initial information common to all device types."""
        self.device_id = device_id
        self.raw = raw
        self.changed_keys = set()

        self.loop = loop
        self._async_set_callback = async_set_callback
        self._cancel_retry = None

        self._async_callbacks = []
        _LOGGER.debug('%s created as \n%s', self.name, pformat(self.raw))

    async def async_set_config(self, data):
        """Set config of device."""
        field = self.deconz_id + '/config'
        await self.async_set(field, data)

    async def async_set_state(self, data: dict):
        """Set state of device."""
        field = self.deconz_id + '/state'
        await self.async_set(field, data)

    async def async_set(self, field, data, tries=0):
        """Set state of device."""
        self.cancel_retry()

        try:
            await self._async_set_callback(field, data)

        except BridgeBusy:
            _LOGGER.debug("BridgeBusy, schedule retry %s %s", field, str(data))

            def retry_set():
                """Retry set state."""
                self._cancel_retry = None
                self.loop.create_task(self.async_set(field, data, tries + 1))

            if tries < 3:
                retry_delay = 2 ** (tries + 1)
                self._cancel_retry = \
                    self.loop.call_later(retry_delay, retry_set)

    def cancel_retry(self):
        if self._cancel_retry is not None:
            self._cancel_retry.cancel()
            self._cancel_retry = None

    def register_async_callback(self, async_callback):
        """Register callback for signalling.

        Callback will be called at the end of
        updating device information in self.async_update.
        """
        self._async_callbacks.append(async_callback)

    def remove_callback(self, callback):
        """Remove callback previously registered."""
        if callback in self._async_callbacks:
            self._async_callbacks.remove(callback)

    def async_update(self, raw):
        """Update input attr in self.

        Store a set of keys with changed values.
        """
        changed_keys = set()

        for k, v in raw.items():
            changed_keys.add(k)

            if isinstance(self.raw.get(k), dict) and isinstance(v, dict):
                changed_keys.update(set(v.keys()))
                self.raw[k].update(v)

            else:
                self.raw[k] = v

        self.changed_keys = changed_keys

        for async_signal_update in self._async_callbacks:
            async_signal_update()

    @property
    def deconz_id(self):
        """Id to call device over API e.g. /sensors/1."""
        return self.DECONZ_TYPE + self.device_id

    @property
    def etag(self):
        """HTTP etag which changes on any action to the device."""
        return self.raw.get('etag')

    @property
    def manufacturer(self):
        """The manufacturer of the device."""
        return self.raw.get('manufacturername')

    @property
    def modelid(self):
        """An identifier unique to the product."""
        return self.raw.get('modelid')

    @property
    def name(self):
        """Name of the device."""
        return self.raw.get('name')

    @property
    def swversion(self):
        """Firmware version."""
        return self.raw.get('swversion')

    @property
    def type(self):
        """Human readable type of the device."""
        return self.raw.get('type')

    @property
    def uniqueid(self):
        """The current state of the device."""
        return self.raw.get('uniqueid')



"""Python library to connect deCONZ and Home Assistant to work together."""

import logging

from .deconzdevice import DeconzDevice

_LOGGER = logging.getLogger(__name__)


class DeconzGroup(DeconzDevice):
    """deCONZ light group representation.

    Dresden Elektroniks documentation of light groups in deCONZ
    http://dresden-elektronik.github.io/deconz-rest-doc/groups/
    """

    DECONZ_TYPE = '/groups/'

    def __init__(self, device_id, raw, loop, async_set_state_callback):
        """Set initial information about light group.

        Create scenes related to light group.
        """
        super().__init__(device_id, raw, loop, async_set_state_callback)
        self._scenes = {}
        self.async_add_scenes(raw.get('scenes'), async_set_state_callback)

    async def async_set_state(self, data):
        """Set state of light group."""
        field = self.deconz_id + '/action'
        await self.async_set(field, data)

    @property
    def state(self):
        """True if any light in light group is on."""
        return self.any_on

    @property
    def brightness(self):
        """Brightness of the light.

        Depending on the light type 0 might not mean visible "off"
        but minimum brightness.
        """
        return self.raw['action'].get('bri')

    @property
    def ct(self):
        """Mired color temperature of the light. (2000K - 6500K)."""
        return self.raw['action'].get('ct')

    @property
    def hue(self):
        """Color hue of the light.

        The hue parameter in the HSV color model is between 0°-360°
        and is mapped to 0..65535 to get 16-bit resolution.
        """
        return self.raw['action'].get('hue')

    @property
    def sat(self):
        """Color saturation of the light.

        There 0 means no color at all and 255 is the greatest saturation
        of the color.
        """
        return self.raw['action'].get('sat')

    @property
    def xy(self):
        """CIE xy color space coordinates as array [x, y] of real values (0..1)."""
        if 'xy' not in self.raw['action'] or \
                self.raw['action']['xy'] == (None, None):
            return None

        x, y = self.raw['action']['xy']

        if x > 1:
            x = x / 65555

        if y > 1:
            y = y / 65555

        return (x, y)

    @property
    def colormode(self):
        """The current color mode of the light.

        hs - hue and saturation
        xy - CIE xy values
        ct - color temperature
        """
        return self.raw['action'].get('colormode')

    @property
    def effect(self):
        """Effect of the light.

        none - no effect
        colorloop
        """
        return self.raw['action'].get('effect')

    @property
    def reachable(self):
        """True if the light is reachable and accepts commands."""
        return True

    @property
    def groupclass(self):
        """"""
        return self.raw.get('class')

    @property
    def all_on(self):
        """True if all lights in light group are on"""
        return self.raw['state'].get('all_on')

    @property
    def any_on(self):
        """True if any lights in light group are on"""
        return self.raw['state'].get('any_on')

    @property
    def devicemembership(self):
        """List of device ids (sensors) when group was created by a device."""
        return self.raw.get('devicemembership')

    @property
    def hidden(self):
        """Indicate the hidden status of the group.

        Has no effect at the gateway but apps can uses this to hide groups.
        """
        return self.raw.get('hidden')

    @property
    def id(self):
        """The id of the group."""
        return self.raw.get('id')

    @property
    def lights(self):
        """A list of all light ids of this group.

        Sequence is defined by the gateway.
        """
        return self.raw.get('lights')

    @property
    def lightsequence(self):
        """A list of light ids of this group that can be sorted by the user.

        Need not to contain all light ids of this group.
        """
        return self.raw.get('lightsequence')

    @property
    def multideviceids(self):
        """A list of light ids of this group.

        Subsequent ids from multidevices with multiple endpoints.
        """
        return self.raw.get('multideviceids')

    @property
    def scenes(self):
        """A list of scenes of the group."""
        return self._scenes

    def async_add_scenes(self, scenes, async_set_state_callback):
        """Add scenes belonging to group."""
        self._scenes = {
            scene['id']: DeconzScene(self, scene, async_set_state_callback)
            for scene in scenes
            if scene['id'] not in self._scenes
        }

    def update_color_state(self, light):
        """Sync color state with light."""
        self.async_update({
            'action': {
                'bri': light.brightness,
                'hue': light.hue,
                'sat': light.sat,
                'ct': light.ct,
                'xy': light.xy or (None, None),
                'colormode': light.colormode,
            },
        })


class DeconzScene:
    """deCONZ scene representation.

    Dresden Elektroniks documentation of scenes in deCONZ
    http://dresden-elektronik.github.io/deconz-rest-doc/scenes/
    """

    def __init__(self, group, scene, async_set_state_callback):
        """Set initial information about scene.

        Set callback to set state of device.
        """
        self._group_id = group.id
        self._group_name = group.name
        self._id = scene.get('id')
        self._name = scene.get('name')
        self._deconz_id = group.deconz_id + '/scenes/' + self._id
        self._async_set_state_callback = async_set_state_callback

    async def async_set_state(self, data):
        """Recall scene to group."""
        field = self._deconz_id + '/recall'
        await self._async_set_state_callback(field, data)

    @property
    def deconz_id(self):
        """Id to call scene over API e.g. /groups/1/scenes/1."""
        return self._deconz_id

    @property
    def full_name(self):
        """Full name."""
        return self._group_name + ' ' + self._name

    @property
    def id(self):
        """Scene ID from deCONZ."""
        return self._id

    @property
    def name(self):
        """Scene name."""
        return self._name



"""Python library to connect deCONZ and Home Assistant to work together."""

import logging

_LOGGER = logging.getLogger(__name__)


class DeconzConfig:
    """deCONZ configuration representation.

    Dresden Elektroniks documentation of config in deCONZ
    http://dresden-elektronik.github.io/deconz-rest-doc/config/
    """

    def __init__(self, raw):
        """Set configuration about deCONZ gateway."""
        self.raw = raw

    @property
    def apiversion(self):
        """The version of the deCONZ Rest API."""
        return self.raw.get('apiversion')

    @property
    def bridgeid(self):
        """Hardware ID."""
        return self.raw.get('bridgeid')

    @property
    def linkbutton(self):
        """True if the gateway is unlocked."""
        return self.raw.get('linkbutton')

    @property
    def mac(self):
        """MAC address of the gateway"""
        return self.raw.get('mac')

    @property
    def modelid(self):
        return self.raw.get('modelid')

    @property
    def name(self):
        """Name of the gateway."""
        return self.raw.get('name')

    @property
    def networkopenduration(self):
        """Can be used to store the permitjoin value permanently."""
        return self.raw.get('networkopenduration')

    @property
    def panid(self):
        """The ZigBee pan ID of the gateway."""
        return self.raw.get('panid')

    @property
    def swversion(self):
        """The software version of the gateway."""
        return self.raw.get('swversion')

    @property
    def uuid(self):
        """UPNP Unique Id of the gateway."""
        return self.raw.get('uuid')

    @property
    def websocketport(self):
        """Websocket port."""
        return self.raw.get('websocketport')

    @property
    def zigbeechannel(self):
        """The current wireless frequency channel used by the Gateway.

        Supported channels: 11, 15, 20, 25."""
        return self.raw.get('zigbeechannel')
